/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 8-5 - buttonup.c - Using client_data to pass app structs

 *  HDR2: Chapter 8, page 188
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/***  Include Required Header Files    ******************************/

#include <X11/Intrinsic.h>  /* The Intrinsics                       */
#include <Xm/Xm.h>          /* Resource symbols                     */
#include <Xm/RowColumn.h>   /* Motif XmRowColumn                    */
#include <Xm/PushB.h>       /* Motif XmPushButton                   */
#include <Xm/TextF.h>       /* Motif XmTextField                    */

/***  Typedef for Client Data  **************************************/

typedef struct {
  Widget textf;             /* Text field widget ID                  */
  int last_button;          /* Last button to call back              */
} AppData;

/***  Forward Declare Callback Routines  *****************************/

void OneCallback();         /* Callback routine                      */

/****** Start of main() **********************************************/

main(argc,argv)
int  argc;
char **argv;
{

 AppData appdata;
 XtAppContext context;
 Widget toplevel, rowcol, textf, btn1, btn2, btn3;

 toplevel = XtAppInitialize(&context, "Buttons", NULL, 0, 
     &argc, argv, NULL, NULL, 0);

 rowcol = XtCreateManagedWidget( "rowcol", xmRowColumnWidgetClass,
     toplevel, NULL, 0);

 textf = XtVaCreateManagedWidget( "textf", xmTextFieldWidgetClass,
     rowcol, XmNvalue, "Press in Sequence", NULL);

 btn1 = XtVaCreateManagedWidget( "One", xmPushButtonWidgetClass, 
     rowcol, XmNuserData, (XtPointer) 1, NULL );

 btn2 = XtVaCreateManagedWidget( "Two", xmPushButtonWidgetClass, 
     rowcol, XmNuserData, (XtPointer) 2, NULL );

 btn3 = XtVaCreateManagedWidget( "Three", xmPushButtonWidgetClass, 
     rowcol, XmNuserData, (XtPointer) 3, NULL );

 appdata.textf = textf;   /**** Initialize application data **********/
 appdata.last_button = 0;

/***** Register callbacks with same client_data **********************/

 XtAddCallback(btn1, XmNactivateCallback, OneCallback,
     (XtPointer)&appdata );
 XtAddCallback(btn2, XmNactivateCallback, OneCallback,
     (XtPointer)&appdata );
 XtAddCallback(btn3, XmNactivateCallback, OneCallback,
     (XtPointer)&appdata );

/****** Realize widget tree and start event loop ********************/

 XtRealizeWidget(toplevel);
 XtAppMainLoop(context);
}

/****** Callback Routines *******************************************/

void OneCallback( w, client_data, call_data )
Widget w;
XtPointer client_data;                    /* Holds application data */
XtPointer call_data;
{
 int index;
 static char *output_strings[] = { "First", "Second", "Third"};
 AppData* adp = (AppData*)client_data;

 XtVaGetValues( w, XmNuserData, &index, NULL);

 if (index == adp->last_button+1) {
     XtVaSetValues( adp->textf, 
         XmNvalue, output_strings[index-1], 
         NULL);
     if (index == 3) adp->last_button = 0;
     else adp->last_button = index;
 }

 else XBell(XtDisplay(w), 50);

}
