/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 7-2 - complex.c - Complex layout w/nested manager widgets
 *  
 *  HDR2: Chapter 7, page 160
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/***  Include Required Header Files  *******************************/

#include <X11/Intrinsic.h>     /* The Intrinsics                   */
#include <Xm/Xm.h>             /* Motif symbols                    */
#include <Xm/MainW.h>          /* XmMainWindow widget              */
#include <Xm/ScrolledW.h>      /* XmScrolledWindow widget          */
#include <Xm/RowColumn.h>      /* XmRowColumn widget               */
#include <Xm/Form.h>           /* XmForm widget                    */
#include <Xm/Text.h>           /* XmText widget                    */
#include <Xm/CascadeB.h>       /* XmCascadeButton widget           */
#include <Xm/PushB.h>          /* XmPushButton widget              */
#include <Xm/DrawingA.h>       /* XmDrawingArea widget             */

/****  Declare Global Variables  ***********************************/

XtAppContext  context;         /* Application context              */

Widget     toplevel, manager, sub_layout;
Widget     menu_bar, menu_button[5];
Widget     control_panel, button_panel, panel_button[6],
           msg_text, msg_scroll;
Widget     graphics, graphics_scroll;

int i;

/***  Main Program    **********************************************/

main(argc,argv)
int argc;
char **argv;
{

/***  Init. Toolkit, App. Context, Application Shell   *************/

  toplevel = XtAppInitialize( &context, "Complex",
      NULL, 0, &argc, argv, NULL, NULL, 0); 

/***  Create widget tree, bind callbacks  **************************/

  manager = XtCreateManagedWidget( "manager",
      xmMainWindowWidgetClass, toplevel, NULL, 0);

  menu_bar = XtCreateManagedWidget( "menuBar",      /* menu bar    */
      xmRowColumnWidgetClass, manager, NULL, 0);
  for(i=0; i<5; i++) {          /* menu buttons*/
    menu_button[i] = XtCreateManagedWidget( "menuButton",
      xmCascadeButtonWidgetClass, menu_bar, NULL, 0);
  }

  sub_layout = XtCreateManagedWidget( "subLayout",  /* form        */
      xmFormWidgetClass, manager, NULL, 0);

  graphics_scroll = XtCreateManagedWidget( "graphicsScroll",
      xmScrolledWindowWidgetClass, sub_layout, NULL, 0);
  graphics = XtCreateManagedWidget( "graphics",
      xmDrawingAreaWidgetClass, graphics_scroll, NULL, 0);

  control_panel = XtCreateManagedWidget( "controlPanel",
      xmFormWidgetClass, sub_layout, NULL, 0);

  button_panel = XtCreateManagedWidget( "buttonPanel",
      xmRowColumnWidgetClass, control_panel, NULL, 0);
  for(i=0; i<6; i++) {
    panel_button[i] = XtCreateManagedWidget( "button",
        xmPushButtonWidgetClass, button_panel, NULL, 0);
  }

  msg_scroll = XtCreateManagedWidget( "msgScroll",
      xmScrolledWindowWidgetClass, control_panel, NULL, 0);
  msg_text =  XtCreateManagedWidget( "msgText",
      xmTextWidgetClass, msg_scroll, NULL, 0);

/***   Tell XmMainWindow about children   **************************/

  XtVaSetValues(manager, 
      XmNworkWindow, sub_layout, 
      XmNmenuBar, menu_bar, 
      NULL );

/***    Attach widgets in XmForm     *******************************/

  XtVaSetValues(graphics_scroll, 
      XmNleftAttachment, XmATTACH_WIDGET, 
      XmNleftWidget, control_panel, 
      NULL);

/***    Attach widgets in XmForm     *******************************/

  XtVaSetValues(msg_scroll, 
      XmNtopAttachment, XmATTACH_WIDGET, 
      XmNtopWidget, button_panel, 
      NULL );

  XtRealizeWidget(toplevel);           /* Realize all              */
  XtAppMainLoop(context);              /* Can't get past here !!!  */
}

/***********    end of main    *************************************/

