/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 10-1 - sender.c - Sending messages via window properties
 *  
 *  HDR2: Chapter 10, page 287
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/*****  Include Required Header Files    *****************************/

#include <X11/Intrinsic.h>        /* The Intrinsics                  */
#include <X11/Xatom.h>            /* X11 Predefined atoms            */
#include <Xm/Xm.h>                /* Motif resource symbols          */
#include <Xm/RowColumn.h>         /* Motif XmRowColumn widget        */
#include <Xm/Label.h>             /* Motif XmLabel widget            */
#include <Xm/Text.h>              /* Motif XmText widget             */
#include <Xm/PushB.h>             /* Motif XmPushButton widget       */

/***** Declare global variables  *************************************/

XtAppContext  context;
Widget    toplevel, manager, label, text, exit_btn;

Display   *display;               /* Display pointer                 */
Window    root_window;            /* Root window ID                  */
int       status;                 /* Returned status value           */

Window    receiver_window;        /* Window ID of receiver app shell */

Atom      RECEIVER_WINDOW,        /* Property to hold receiver win ID*/
          PRIVATE_MESSAGE,        /* Identifies ClientMessage type   */
          PRIVATE_PROPERTY;       /* Property in which to store msg  */

XtIntervalId limit_timer;         /* Callback ID - message timeout   */
Boolean      timed_out;           /* True if message timed out       */

/***  Forward declare callback routines ******************************/

Window GetReceiverWin();          /* Function - get receiver win ID  */
void CheckReceiverWin();          /* Function - handle receiver exit */
void ExitRoutine();               /* Callback - exits application    */
void SendMessage();               /* Callback - write prop & send msg*/
void VerifyReceipt();             /* Handler - verify prop deleted   */
void TimedOut() ;                 /* Timer - timeout for msg xfer    */

/***  Main program  **************************************************/

main(argc,argv)
int argc;
char **argv;
{

 XEvent  event;                   /* Last event received             */

 toplevel = XtVaAppInitialize( &context,  "Sender", 
     NULL, 0, &argc, argv, NULL, NULL );
 XtAddEventHandler(toplevel, PropertyChangeMask, False, 
     VerifyReceipt, NULL );

 manager = XtCreateManagedWidget("manager", 
     xmRowColumnWidgetClass, toplevel, NULL, 0);

 label = XtCreateManagedWidget("label",
     xmLabelWidgetClass, manager, NULL, 0);

 text = XtCreateManagedWidget("text",
     xmTextWidgetClass, manager, NULL, 0);
 XtAddCallback( text, XmNactivateCallback,
     SendMessage, NULL );

 exit_btn = XtCreateManagedWidget("exitBtn",
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback( exit_btn, XmNactivateCallback,
     ExitRoutine, NULL );

/* Realize all; Get display pointer and root window ID */

 XtRealizeWidget(toplevel);     

 display = XtDisplay(toplevel);
 root_window = XRootWindowOfScreen(XtScreen(toplevel));

 /* See if receiver.c has already interned atom */

 RECEIVER_WINDOW = XInternAtom(display, "RECEIVER_WINDOW", True);  

 if (RECEIVER_WINDOW == NULL) {
    printf("Receiver is not running\n");    /* If not, receiver is   */
    exit(0);                                /* not running so exit   */
 }

 /* Get receiver window ID; ask to be notified if it changes  */

 receiver_window = GetReceiverWin(toplevel);
 XSelectInput(display, root_window, PropertyChangeMask);      

/* Intern Atoms to identify client message and message property name */ 

 PRIVATE_MESSAGE = XInternAtom(display, "PRIVATE_MESSAGE", False);   
 PRIVATE_PROPERTY = XInternAtom(display, "PRIVATE_PROPERTY", False);

/* Custom main loop to check for events outside widget tree          */
/* If root window property changes, maybe there is a new receiver    */

 while(1) {
   XtAppNextEvent(context, &event);
   if ((event.type==PropertyNotify)&&(event.xproperty.window==root_window)) {
       CheckReceiverWin(&event);
   }
   else { 
       XtDispatchEvent(&event);            /* Dispatch widget events */
   }
 }
}

/*** Application callback routines ***********************************/

void                                       /* Called to exit         */
ExitRoutine(w, client_data, call_data)
Widget w;                                  /* Unused                 */
XtPointer client_data, call_data;          /* Unused                 */
{
  exit();
}


void 
SendMessage(w, client_data, call_data)     /* Put string in property */
Widget w;                                  /* & send ClientMessage   */
XtPointer client_data, call_data;
{
 String  message_string;                   /* message string         */
 XClientMessageEvent synth_event;          /* Client message struct  */

 /* get the message string and place in property; prevent new input  */

 XtVaGetValues(text, XmNvalue, &message_string, NULL);
 XtVaSetValues(text, XmNeditable, False, NULL);

 status = XChangeProperty(
     display,                           /* Display shared w receiver */
     XtWindow(toplevel),                /* Attach to Appl Shell      */
     PRIVATE_PROPERTY,                  /* Atom = property name      */
     XA_STRING,                         /* Atom = property type      */
     8,                                 /* Format is 8-bit values    */
     PropModeReplace,                   /* Replace any previous value*/
     (char*)message_string,             /* Message value             */
     strlen(message_string)+1);         /* # of 8-bit elements       */

 /* synthesize an event to tell receiver about message */

 synth_event.type = ClientMessage;
 synth_event.message_type = PRIVATE_MESSAGE;
 synth_event.display = display;
 synth_event.window = receiver_window;
 synth_event.format = 32;
 synth_event.data.l[0] = PRIVATE_PROPERTY;
 synth_event.data.l[1] = XtWindow(toplevel);
 synth_event.data.l[2] = strlen(message_string)+1;

 XSendEvent(display, receiver_window, True, NoEventMask, (XEvent*)&synth_event);

 /* Set timeout on message transfer */
 limit_timer = XtAppAddTimeOut( context, 5000, TimedOut, NULL);
 timed_out = False;

 XtVaSetValues(text, XmNvalue, "Message Sent", NULL);

}

/*********************************************************************/

void 
VerifyReceipt(w, client, event, dispatch) /* If property deleted   */
Widget w;                                 /* message was received  */
XtPointer client;
XEvent *event;
Boolean *dispatch;
{

  /* If property was deleted by receiver, message was received */

  if ( event->type == PropertyNotify
         && event->xproperty.state == PropertyDelete
         && event->xproperty.atom == PRIVATE_PROPERTY 
         && !timed_out ) {

      XtVaSetValues(text, XmNvalue, "Receipt Verified", NULL);
      XtVaSetValues(text, XmNeditable, True, NULL);

      /* cancel timer if it has not expired */
      if (limit_timer) XtRemoveTimeOut(limit_timer);
  }
}

/*********************************************************************/

void 
CheckReceiverWin(event)          /* Handle changes in receiver win   */
XEvent *event;
{
 /* Look for the right event */
 if ((event->type != PropertyNotify)
    || (event->xproperty.atom != RECEIVER_WINDOW)) return;


 if (event->xproperty.state == PropertyDelete){
    printf ("receiver exited\n");
    exit(1);
 }
 else receiver_window = GetReceiverWin(toplevel);
}

/*********************************************************************/

Window
GetReceiverWin()                   /* Return receiver window ID from  */
{                                  /* root window property            */
 Atom type_return; 
 int  format_return;
 unsigned long bytes_left, nitems_return;

 Window *value;                    /* Expected value is window ptr    */

 status = XGetWindowProperty( 
     display,                      /* display shared with receiver    */
     root_window,                  /* window with property to read    */
     RECEIVER_WINDOW,              /* Atom = name of property to read */
     0,                            /* Offset at which to start read   */
     1,                            /* # of 32-bit items to read       */
     False,                        /* Don't delete after reading      */
     XA_WINDOW,                    /* Requested type                  */
     &type_return,                 /* Atom = actual type found        */
     &format_return,               /* Returned element format         */
     &nitems_return,               /* # of returned elements          */
     &bytes_left,                  /* Should be 0                     */
     (unsigned char**)&value);     /* Returned value of property      */

 if ((status!=Success) | (value == NULL)){
     printf("Can't find receiver window\n");
     exit(1);
 }

 return (Window)*value;                 /* Return window ID           */

}

/**********************************************************************/

void 
TimedOut(client, timerid)               /* Message was not picked up  */
XtPointer client;                       /* so time out                */
XtIntervalId timerid;
{
  /* Message not received - delete property, prepare for new input    */

  XtVaSetValues(text, XmNvalue, "Timed Out", NULL);
  XtVaSetValues(text, XmNeditable, True, NULL);

  XDeleteProperty( display, XtWindow(toplevel), PRIVATE_PROPERTY );
  limit_timer = 0;
  timed_out = True;
}

