/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 9-3 - translate.c - Adding translations to a widget
 *  
 *  HDR2: Chapter 9, page 257
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/***  Include required header files   ********************************/

#include <X11/Intrinsic.h>        /* The Intrinsics                  */
#include <Xm/Xm.h>                /* Motif resource symbols          */
#include <Xm/Text.h>              /* for XmStringCreateLocalized     */
#include <Xm/BulletinB.h>         /* Motif XmRowColumn widget        */
#include <Xm/Label.h>             /* Motif XmLabel widget            */

/***** Declare global variables  ************************************/

#define MAX_CHARS 200
XtAppContext context;             /* Application context             */
Widget toplevel, manager, label;  /* Widget tree                     */
char   label_string[MAX_CHARS];   /* Input character buffer          */
int    label_length = 0;          /* Length of current input         */

/*****  Declare action procedures, translation table/action list  ****/

void ExitAction();      /* Action proc - Exits                       */
void ClearText();       /* Action proc - Clears label string         */
void AddKeystroke();    /* Action proc - Adds a character to string  */
void Highlight();       /* Action proc - Reverses fg and bg colors   */
void TakeFocus();       /* Action proc - Takes keyboard focus        */

XtTranslations translations;       /* Compiled translations          */

static char translation_table[] = 
    "Ctrl<Key>x:          ClearText() \n\
     Ctrl<Key>d:          ExitAction() \n\
     <Key>:               AddKeystroke() \n\
     <Btn1Down>,<Btn1Up>: TakeFocus() \n\
     <Btn1Up>(2):         Highlight()";

static XtActionsRec actions[] = {  /* Action list                    */
    {"ClearText",    ClearText},
    {"AddKeystroke", AddKeystroke},
    {"Highlight",    Highlight},
    {"ExitAction",   ExitAction},
    {"TakeFocus",    TakeFocus}
    };

/***  Main Program  ***************************************************/

main(argc,argv)
int argc;
char **argv;
{

 toplevel = XtAppInitialize(&context, "Translate", 
     NULL, 0, &argc, argv, NULL, NULL, 0);

 manager = XtCreateManagedWidget("manager", 
     xmBulletinBoardWidgetClass, toplevel, NULL, 0);

 label = XtCreateManagedWidget("label",
     xmLabelWidgetClass, manager, NULL, 0);

 /* Register global actions      */
 XtAppAddActions(context, actions, XtNumber(actions));

 /* compile translation table and augment XmLabel translations  */
 translations = XtParseTranslationTable(translation_table);
 XtAugmentTranslations(label, translations);

 XtRealizeWidget(toplevel);        /* Realize all & bind actions     */
 XtAppMainLoop(context);           /* Start event loop               */
}

/*** Action Procedures ***********************************************/

void TakeFocus(w, event, strings, num_strings)
Widget w;                         /* The label widget                */
XEvent *event;                    /* Unused                          */
String *strings;                  /* Unused                          */
Cardinal *num_strings;            /* Unused                          */
{
    XmProcessTraversal(w, XmTRAVERSE_CURRENT);
}

/*********************************************************************/

void AddKeystroke(w, event, strings, num_strings)
Widget w;                        /* The label widget                 */
XEvent *event;                   /* A Key event                      */
String *strings;                 /* Unused                           */
Cardinal *num_strings;           /* Unused                           */
{

  char     new_char[2];          /* Input as ASCII char              */
  KeySym   new_keysym;           /* Input as KeySym                  */
  XmString xmstring;             /* Input converted to XmString      */
  int nbytes;

  /* Get ASCII character with XLookupString - this is not            */
  /* a comprehensive solution but works OK for Latin-1 keyboards     */

  nbytes = XLookupString((XKeyEvent*)event, new_char,
      1, &new_keysym, (XComposeStatus*)NULL);

  new_char[1]=(char)NULL;

  if ( (nbytes > 0) && (label_length < (MAX_CHARS-1)) ){
      strcat(label_string, new_char);          /* Add to string      */
      label_length = strlen(label_string);

      /* Convert to Motif compound string and set label string       */
      xmstring = XmStringCreateLocalized(label_string);
      XtVaSetValues(w, XmNlabelString, xmstring, NULL);
      XmStringFree(xmstring);
  }
}


/*********************************************************************/

void 
ClearText(w, event, strings, num_strings)
Widget w;                       /* The label widget                  */
XEvent *event;                  /* Unused                            */
String *strings;                /* Unused                            */
Cardinal *num_strings;          /* Unused                            */
{
  /* clear the label string and input buffer, and reset counters     */

  XmString xmstring = XmStringCreateLocalized("");
  XtVaSetValues(w, XmNlabelString, xmstring, NULL);
  XmStringFree(xmstring);

  label_string[0] = (char)NULL;
  label_length = 0;
}

/*********************************************************************/

void 
Highlight(w, event, strings, num_strings)
Widget w;                       /* The label widget                  */
XEvent *event;                  /* Unused                            */
String *strings;                /* Unused                            */
Cardinal *num_strings;          /* Unused                            */
{
  /* Swap the foreground and background colors */

  Pixel fg, bg;

  XtVaGetValues(w, XmNforeground, &fg, XmNbackground, &bg, NULL);
  XtVaSetValues(w, XmNforeground, bg, XmNbackground, fg, NULL);
}

/*********************************************************************/

void 
ExitAction(w, event, strings, num_strings)
Widget w;                       /* The label widget                  */
XEvent *event;                  /* Unused                            */
String *strings;                /* Unused                            */
Cardinal *num_strings;          /* Unused                            */
{
  exit();                       /* exit                              */
}
