/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 9-1 - vote.c - Opening windows on multiple displays
 *  
 *  HDR2: Chapter 9, page 227, with error handler from 
 *                             Code Fragment 13-1, page 405
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/*****  Include Required Header Files    *****************************/

#include <X11/Intrinsic.h>          /* The Intrinsics                */
#include <Xm/Xm.h>                  /* Resource symbols              */
#include <Xm/RowColumn.h>           /* Motif XmRowColumn             */
#include <Xm/Label.h>               /* Motif XmLabel                 */
#include <Xm/PushB.h>               /* Motif XmPushButton            */
#include <setjmp.h>                 /* setjmp/longjmp header file    */

/*****  Declare Global Variables   ***********************************/

int yes_votes=0, no_votes=0;        /* tally of votes                */

void TallyVotes();                  /* Callback to log votes         */
void EndVoting();                   /* Routine to tally votes        */
void IODisconnectHandler();         /* I/O error handler             */

/*****  Command Line Options   ***************************************/

static XrmOptionDescRec options[] = {
    {"-q", "*question.labelString", XrmoptionSepArg, NULL }
  };

/*****  Declare Fallback Resources   *********************************/

static String fallback_resources[]= {
  "*question.labelString: Please select yes or no:",
  "*main.entryAlignment:  ALIGNMENT_CENTER",
  NULL };

jmp_buf restore_buf;                /* container for stack env.      */

/***** The display list should probably be an input argument  ********/

static String dispnames[] = {        /* A list of displays            */
"widget:0.0",                        /* You'll have to modify this to */
"widget:0.0"};                       /* use your own machines ;-)     */

int ndisp;
Display **display;

/*****  Start of main()   ********************************************/

main(argc, argv)
int argc;
char **argv;
{
  int i, n;
  XtAppContext context;
  Widget toplevel, main, question, yes, no;

/* save argc and argv and initialize toolkit */

  int argc_orig = argc;
  char **argv_orig = (char**)malloc(argc * sizeof(char*));
  for (i=0; i<argc; i++) argv_orig[i] = argv[i];

/*****  Init. Toolkit, Appl. Context, Application Shell  *************/

  XtToolkitInitialize();
  context = XtCreateApplicationContext();
  XtAppSetFallbackResources(context, fallback_resources);

  ndisp = XtNumber(dispnames);
  display = (Display**) XtMalloc( ndisp * sizeof(Display*) );

  /* loop to open multiple displays and build widget trees */
  for (n=0; n < ndisp; n++) {

     /* restore argc and argv and open next display  */
     argc = argc_orig;
     for (i=0; i<argc; i++) argv[i] = argv_orig[i];

     display[n] = XtOpenDisplay(context, dispnames[n], "vote", "Vote", 
          options, XtNumber(options), &argc, argv);

     if (display[n]) {
          /* In case window manager asks us politely to close, just
             unmap rather than exiting */
          toplevel = XtVaAppCreateShell("vote", "Vote",
              applicationShellWidgetClass, display[n], 
              XmNdeleteResponse, XmUNMAP, NULL);
          main = XtCreateManagedWidget("main", xmRowColumnWidgetClass,
              toplevel, NULL, 0 );
          question = XtCreateManagedWidget("question", xmLabelWidgetClass,
              main, NULL, 0 );
          yes = XtCreateManagedWidget("yes", xmPushButtonWidgetClass, 
              main, NULL, 0 );
          XtAddCallback(yes, XmNactivateCallback, TallyVotes,
              (XtPointer)True);
          no = XtCreateManagedWidget("no", xmPushButtonWidgetClass, 
               main, NULL, 0 );
          XtAddCallback(no, XmNactivateCallback, TallyVotes, 
               (XtPointer)False);
          XtRealizeWidget(toplevel);
      }
  }

  XtAppAddTimeOut(context, 30000, EndVoting, NULL);

  /* If something rudely axes the server connection, the 
     XIOErrorHandler registered below will save us */
  /* Save stack state in case we need to jump back here */
  if (setjmp(restore_buf) == 0)
      XSetIOErrorHandler(IODisconnectHandler);
      
  XtAppMainLoop(context);
}

/*** Callback routines and utilities *********************************/

void TallyVotes( w, client, call)    /* Keep running count of votes  */
Widget w;
XtPointer client, call;
{
  if ((Boolean)client == True ) 
      yes_votes++;
  else 
      no_votes++;

  while (XtParent(w)) 
      w = XtParent(w);               /* Climb up to toplevel         */

  XtDestroyWidget(w);                /* Remove from screen           */

  if ((yes_votes+no_votes) == XtNumber(dispnames)) 
      EndVoting();
}

/*********************************************************************/

void EndVoting()                       /* Print out results and exit */
{
  printf("Voting over; there were:\n");
  printf("\t%i\tYes votes\n", yes_votes);
  printf("\t%i\tNo votes\n", no_votes);
  printf("\t%i\tAbstentions\n", XtNumber(dispnames)-(yes_votes+no_votes));
  exit(1);
}

/*********************************************************************/

void IODisconnectHandler(bad_display)
Display *bad_display;
{
  int i;
  
  /* This I/O error handler responds to the loss of a server connection
     by logging a message and then performing a longjmp() to avoid the
     exit that is otherwise enforced by the X internals */
  printf("display fucked\n");     
  /* Report display problem */
  for (i=0; i<ndisp; i++){
    if(bad_display== display[i]) {
       XtAppWarning( XtDisplayToApplicationContext(bad_display), 
           "Display %s disconnected\n", dispnames[i]);
    }
  }
  /* Tidy up after broken connecttion if necessary */
  /* Then restore stack to earlier state */
  longjmp(restore_buf, 1);
}

